pub struct CaptureSpanAndStackTrace;Expand description
Implementation of CaptureTrace that captures both a span trace using tracing_error::SpanTrace
and a stack trace.
tracing must be initialized with tracing_error::ErrorLayer for the span trace to be captured successfully
and RUST_BACKTRACE environment variable must be set for the stack trace to be captured.